//
//  ConverterController.m
//  CFAbsoluteTimeConverter.app
//
//  Created by John C. Daub <mailto:hsoi@hsoi.com> on Wed Nov 05 2003.
//  No copyright nor copyleft. This is in the public domain.
//
//	This is the ConverterController.m file from the CFAbsoluteTimeConverter.app project. Originally I created
//	CFAbsoluteTimeConverter as a little command line tool, but decided to spruce it up with a GUI
//	instead. What follows are the comments from the command line tool so I don't have to retype it. 

// Simple little Foundation Tool to convert a double (a CFAbsoluteTime) to a human-readable output.
// Written by John C. Daub <mailto:hsoi@hsoi.com> on 4 November 2003 and released into the public
// domain because this is pretty simple stuff and well... share and enjoy. Besides, I don't feel
// like officially maintaining it. :-)
//
// On 4 November 2003 I was contacted by Jeff Mao, Director of Technology for the Maine School district
// (M.S.A.D. #60, in North Berwick, Maine). As a part of the Maine Learning Technology Initiative
// they're equiping every 7th and 8th grader across the state with iBook's (some 38,000). Well, seems some
// kiddos are surfing to sites that the school district would rather them not surf. So the admins are
// looking in the history files for Safari to try to find out when those sites were visited. Safari's
// history file stores the "lastVisitedDate" for a site as some sort of floating point number.
// Since it's Safari, it seems to be using the WebHistory class that's part of the Cocoa WebKit
// framework. WebHistory seems to store dates using NSCalendarDate, and I know from experience
// that NSCalendarDate is close to CFDate, and the only date format that I'm aware of that
// uses a floating point number is a CFAbsoluteTime. So I tried it... take the floating point
// number from the Safari history file, treat it as a CFAbsoluteTime, convert to a CFGregorianDate,
// then output that data. Seems a correct assumption on my part. Mr. Mao sent me a sample history
// file entry containing the number "89672909.2" and that converted to "The time is - year:2003
// month:11 day:4 hour:21 minute:8 second:29.200000" (the sample seems to have been taken just
// before Mr. Mao emailed me on November 4th).
//
// Anyway, so I just investigated and hacked this up out of curiousity and a desire to help the
// guy out. Again, this is all public domain work. I'm going to be giving the code plus a built
// binary to Mr. Mao and he can do with it as he pleases. I'm just a programmer... if those 7th
// and 8th graders don't like what's going on well, either get better about cleaning up after
// yourself (Safari has that "Reset Safari" menu item, y'know) or don't visit the sites to
// begin with. :-)
//
// As for how to use this, it's a command line tool... and a quick hack at that. Just give it
// the time to convert like:
//
//		$ CFAbsoluteTimeConverter 89672909.2
//
// And it'll spit out results. I make no warranties expressed nor implied by any of this. It's
// all in the public domain, it's a hack, and the source is available... so if you don't like
// it, you're welcome to fix it yourself. :-)
//
//
// -------------------------------------------------------------------------
//
// Hsoi 06-Mar-2011 - While this code remains in the public domain, it now maintained and
// kept by Hsoi Enterprises LLC. http://www.hsoienterprises.com  info@hsoienterprises.com


#import "ConverterController.h"

@implementation ConverterController

- (void)awakeFromNib {
	[window center];
	[window makeKeyAndOrderFront:nil];
}


- (IBAction)convertTime:(id)sender
{
#pragma unused(sender)

	[captionConversion setStringValue:@""];
	
	NSString*	numberString = [fieldTime stringValue];
	if (numberString != nil) {
		double value = [numberString doubleValue];
		if (value == HUGE_VAL || value == -HUGE_VAL) {
			[captionConversion setStringValue:@"ERROR: time entered was too large a number."];
		} else if (value == 0.0) {
			[captionConversion setStringValue:@"ERROR: time entered was too small, or wasn't a valid text representation of a floating-point number."];
		} else {
		
			NSTimeInterval	timeInterval = value;

			// Hsoi 03-Mar-2009 - Let's use new OS mechanisms for the date conversion.
	#if 0
			NSCalendarDate*	calDate = [NSCalendarDate dateWithTimeIntervalSinceReferenceDate:timeInterval];
			NSString*		calDateString = [calDate descriptionWithCalendarFormat:@"%A %d %B %Y %I:%M:%S %p"];
			[captionConversion setStringValue:calDateString];
	#else
			NSDate*				convertedDate = [NSDate dateWithTimeIntervalSinceReferenceDate:timeInterval];
			NSDateFormatter*	dateFormatter = [[[NSDateFormatter alloc] init] autorelease];
			[dateFormatter setFormatterBehavior:NSDateFormatterBehavior10_4];
			[dateFormatter setLocale:[NSLocale currentLocale]];
			[dateFormatter setTimeZone:[NSTimeZone localTimeZone]];
			//[dateFormatter setDateFormat:@"%A %d %B %Y %I:%M:%S %p"];
			[dateFormatter setDateStyle:NSDateFormatterFullStyle];
			[dateFormatter setTimeStyle:NSDateFormatterFullStyle];
			
			NSString*	calDateString = [dateFormatter stringFromDate:convertedDate];
			[captionConversion setStringValue:calDateString];
	#endif
		}
	} else {
		[captionConversion setStringValue:@"ERROR: failed to get the time string from the GUI."];
	}
}

- (void)showMainWindow:(id)sender {
#pragma unused(sender)

	[window display];
	[window makeKeyAndOrderFront:nil];
}


- (BOOL)validateMenuItem:(NSMenuItem*)menuItem {
	if ([menuItem action] == @selector(showMainWindow:)) {
		if ([window isVisible]) {
			return NO;
		}
	}
	
	return YES;
}


@end
